// Header file of Ethernet-MIDI-USB converter  ********************************
// for STM32F042F6 or STM32F070F6(20pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
// and WIZ850io wired Ethernet module
//
// Author and (c)2018 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.c", "usb-fs.c", "ether.c" and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org
//
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.0 or 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

// extern defined in userinterface.c  ************
extern uint8_t gcPushByte, gcSetupMode;
extern void  userSetup();
extern void copyComParams(void);

// extern defined in usb-fs.c  ************
extern uint8_t guPushbyte, gcIgnoreVelo0;
extern uint8_t guDeviceClass, guConfigured;
extern void startUsb(void);
extern void stopUSB(void);
extern void initUSB(void);
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbByte(uint8_t);
extern uint32_t pullMidiMsg(void);
extern void enCodeMidi(uint8_t);

// extern defined in ether.c  ************
extern uint8_t geEtherMode;          //reflects S0_MR register of W5500
extern uint32_t geEtherOpen, geDiscount;
extern uint8_t geSPI_RXBUF[];
extern uint8_t *geSPI_RXHEAD, *geSPI_RXTAIL, *geSPI_RXBUFTOP;
extern uint8_t geSPI_TXBUF[];
extern uint8_t *geSPI_TXHEAD, *geSPI_TXBUFTOP;
extern void initSPI1(void);
extern uint8_t exchangeSpiByte(uint8_t);
extern uint8_t initW5500(void);
extern uint8_t openUDP0(void);
extern uint8_t openTCP0(void);
extern void closeTCPConnection(void);
extern void sendEtherByte(uint8_t);
void sendEtherBuffer(uint16_t);
void checkSendEther(void);
extern uint8_t receiveUDP(void);
extern uint8_t receiveTCP(void);
extern void writeWiz_4(uint16_t, uint8_t, uint8_t);
extern uint8_t readWiz_4(uint16_t, uint8_t);
extern uint16_t pullSpiRxBuf(void);

// ****************************************
#define USART_TXBUFLEN 1536
#define USART_RXBUFLEN 256

uint8_t gRXBUF[USART_RXBUFLEN];
volatile uint8_t *gRXHEAD=0,*gRXTAIL=0,*gRXBUFTOP=&(gRXBUF[USART_RXBUFLEN]);
uint8_t gTXBUF[USART_TXBUFLEN];
volatile uint8_t *gTXHEAD=0,*gTXTAIL=0,*gTXBUFTOP=&(gTXBUF[USART_TXBUFLEN]);

//system control
uint16_t gSYSCOUNT=20, gRED_LEDCOUNT, gGREEN_LEDCOUNT;
uint16_t gDELAYCOUNT=0,gTIMEOUT=0, gGUARDTIME, gHGUARDTIME;
uint32_t gStartSetup=0, gHStartSetup, gSetupMode=0, gFlashChanged, gStrobe1ms;

#define PAGE31 (0x08007C00)

struct PARAM{
uint32_t VidPid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t Gateway;
uint32_t LanMode;           //includes BlockF68E configuration
uint32_t UdpDestIP;
uint32_t UdpDestPort;
uint32_t UdpLocalPort;
uint32_t McastIP;
uint32_t McastLocalPort;
uint32_t McastDestPort;
uint32_t TcpPort;
uint32_t DisconTime;
uint32_t ComBaud;
uint32_t MidiBaud;
} gComParam, gComParamShadow ;  //17 words

uint32_t gBlockF6, gBlockFE, gBlockF8;
uint32_t gStreamMaster;         //Ethernet Master= 0(default) , else USB Master

const uint8_t gTypeString [] = "Ether-MIDI-USB Converter v" ;
const uint8_t gCopyString [] = " (c)2018 W.Schemmert";

void msDelay(uint16_t);
void servSystem(void);
void popFlashPage31(void);
uint8_t pushFlashPage31(void);

void initUsart2();
void sendB(uint8_t);
void sendX8(uint8_t);
void sendX16(uint16_t);
void sendNibble(uint8_t);
void sendAllB(uint8_t);
void sendAllString(const uint8_t *);
void sendSetupB(uint8_t);
void sendSetupString(const uint8_t *);
void sendSetupNibble(uint8_t);
void sendSetupX8(uint8_t);
void sendSetupN8(uint8_t);
void sendSetupX16(uint16_t);
void sendSetupX32(uint32_t);
void sendSetupN32(uint32_t);
void sendSetupS32(uint32_t);
void sendSetupS16(uint16_t);
uint16_t pullUsartRxBuf(void);
uint16_t pullSetupRxBuf(void);
void SysTick_Handler(void);
void USART2_IRQHandler(void);
