// Ethernet-MIDI-USB converter  ***********************************************
// for STM32F042F6 or STM32F070F6(20pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
// and WIZ850io wired Ethernet module
//
// Author and (c)2018 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.h", "usb-fs.c", "ether.c" and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx_modified.h" //modifications see top lines of this file
#include "usb_regs.h"
#include "globaldefines.h"
#include "main.h"


int main(void)    // ********************************************************
// **************************************************************************
{
uint16_t recvword;
uint8_t recvbyte;
uint32_t fillstate;

    SCB->CCR |= SCB_CCR_STKALIGN_Msk; // Enable double word stack alignment
//(recommended in Cortex-M3 r1p1, default in Cortex-M3 r2px and Cortex-M4)

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN;      // Enable Port A clock
    RCC->AHBENR |= RCC_AHBENR_GPIOBEN;      // Enable Port B clock

// PA0 is used as red LED output
    GPIOA->MODER |= GPIO_MODER_MODER0_0 ; //PortA0 general purpose output mode
 //   GPIOA->BSRR = GPIO_BSRR_BS_0;         //Set bit PA0 (red LED ON)
       GPIOA->BSRR = GPIO_BSRR_BR_0;         //Set bit PA0 (red LED ON)
// PA1 is used as green LED output
    GPIOA->MODER |= GPIO_MODER_MODER1_0 ; //PortA1 general purpose output mode
  //  GPIOA->BSRR = GPIO_BSRR_BS_1;         //Set bit PA1 (green LED ON)
    GPIOA->BSRR = GPIO_BSRR_BR_1;
    //GPIOx->OTYPER  is push-pull by default
    //GPIOx->OSPEEDR = 0;  //2 MHz by default
    //GPIOx->PUPDR  is HiZ by default

// for USB, remap PA11,12 to PA9,10
    RCC->APB2ENR |= RCC_APB2ENR_SYSCFGCOMPEN;
    SYSCFG->CFGR1 |= SYSCFG_CFGR1_PA11_PA12_RMP;

    gSYSCOUNT=20;
    SysTick_Config(SystemCoreClock/1000);   //1ms / 1kHz SysTick

    gSetupMode = 0;
    gcPushByte = 0;
    gFlashChanged = 0;
    geEtherOpen = 0;
    gGREEN_LEDCOUNT=0;
    gRED_LEDCOUNT=0;
    gDELAYCOUNT=0;
    gTIMEOUT=0;

#ifndef DEBUG
//start Independent Watchdog   *******************
//will be served by "pullUsartRxBuf" and "pullUsbRxBuf"
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x0000FFFF;
    msDelay(2);
    IWDG->PR = 0x00000002;      //resulting timeout ca. 1-2 seconds
    msDelay(2);
    IWDG->KR = 0x0000CCCC;      //start Independent Watchdog
#endif

//check jumper or front panel switch and set guDeviceClass  ******************
//does not conflict with SWDebug when SWDebug cable is not attached
    GPIOA->PUPDR |=  (GPIO_PUPDR_PUPDR13_0 | GPIO_PUPDR_PUPDR14_0);
    GPIOA->PUPDR &=  ~(GPIO_PUPDR_PUPDR13_1 | GPIO_PUPDR_PUPDR14_1);
    msDelay(2);
//if PortA,14 = high (default): CDC_CLASS
//if PortA,14 = low (Jumper set): MIDI_CLASS
    guDeviceClass = CDC_CLASS;
    if ((GPIOA->IDR & m14) == 0)  guDeviceClass = MIDI_CLASS;

//if PortA,13 = high (default): Ethernet is Master of communication with Usart
//if PortA,13 = low (Jumper set): USB is Master of communication with Usart
//"ETHERNET_MASTER" means:
// messages from Usart and USB are merged together to Ethernet
//and messages from Ethernet are sent simultaneously to Usart and USB
//"USB_MASTER" means:
// messages from Usart and Ethernet are merged together to USB
//and messages from USB are sent simultaneously to Usart and Ethernet
//This is NOT a true message merger!! Merge is processed "byte by byte"
//only one "slave" input is allowed be busy during transmission of any message
    gStreamMaster = ETHER_MASTER;
    if ((GPIOA->IDR & m13) == 0)  gStreamMaster = USB_MASTER;

//init preset profile  **************************
    popFlashPage31();           //copy user setup to SRAM for easier access
                                //and perform some system setup
    msDelay(5);

// init communication **************************
    initUsart2();
    msDelay(5);
 //   sendB('#');

    startUsb();
    msDelay(5);

    initSPI1();
  //  sendB('%');
    msDelay(30);

    initW5500();
    msDelay(30);
//    sendX8(geEtherMode);
    if (geEtherMode == 2) openTCP0(); //geEtherMode aleady set by popFlashPage31
    else openUDP0();

    msDelay(30);
//   sendX8(geEtherOpen);
    if (guDeviceClass == CDC_CLASS) sendB(0x0D);
//    if (guConfigured) sendB('$');

    while(1)
    {
        if (gSetupMode) userSetup();        //returns to mainloop after Exit
//start setup mode with 3 consecutive +++ with 1 sec guardtime before & after
//gStartSetup and gHStartSetup work as mutual exclude semaphore, too
        recvword = pullUsartRxBuf();
        if (recvword)  {
            recvbyte = (uint8_t)(recvword & 0x00FF);
            if (guDeviceClass == CDC_CLASS) {
                if (gHStartSetup == 0)  {
                    if ((gStartSetup == 4) && (gGUARDTIME))  {
                        gStartSetup = 0;
                    }
                    if ((recvbyte == '+') && (gStartSetup == 2)) {
                        gStartSetup = 3;
                        gGUARDTIME = 1000;      //milliseconds
                        sendB('+');             //for better visual feedback
                    }
                    if ((recvbyte == '+') && (gStartSetup == 1)) {
                        gStartSetup = 2;
                        gGUARDTIME = 400;
                        sendB('+');
                    }
                    if ((recvbyte == '+')&&(gStartSetup==0)&&(gGUARDTIME==0)) {
                        gStartSetup = 1;
                        gGUARDTIME = 400;
                        sendB('+');
                    }
                    if (gSetupMode == 0) {
                        if(gStreamMaster==ETHER_MASTER) sendEtherByte(recvbyte);
                        else sendUsbByte(recvbyte);
                        if (gStartSetup == 0) gGUARDTIME = 1000;
                    }
                }
                else {
                    if (gStreamMaster==ETHER_MASTER) sendEtherByte(recvbyte);
                    else sendUsbByte(recvbyte);
                }
            }
            else   {
                if (gStreamMaster == ETHER_MASTER) sendEtherByte(recvbyte);
                else enCodeMidi(recvbyte);
            }
#ifdef DEBUG
            if ((recvbyte&0x00FF)==0x7E)  sendDebug();
//Tilde char ~ transmits and flushes the debug buffer.
//It transmits and flushes automatically when it is full.
#endif
        }

//calculate fill level of gUSART_TXBUF
//else interrupt driven USART Tx would overflow at high USB data load
//TAIL>HEAD? then add USART_TXBUFLEN to get the absolute difference
        if (gTXTAIL > gTXHEAD) {
            fillstate = (uint32_t)gTXHEAD + USART_TXBUFLEN - (uint32_t)gTXTAIL;
        }
        else fillstate = (uint32_t)gTXHEAD - (uint32_t)gTXTAIL ;

        if (gStreamMaster == ETHER_MASTER)  {
            if (geEtherOpen && (fillstate < (USART_TXBUFLEN-40))) {
                recvword = pullSpiRxBuf();
                if (recvword)  {
                    recvbyte = (uint8_t)(recvword & 0x00FF);
                    if (guDeviceClass == CDC_CLASS) {
                        if (gStartSetup == 0)  {
                            if ((gHStartSetup == 4) && (gHGUARDTIME)) {
                                gHStartSetup = 0;
                            }
                            if((recvbyte == '+') && (gHStartSetup == 2))  {
                                gHStartSetup = 3;
                                gHGUARDTIME = 1000;      //milliseconds
                                sendEtherByte('+');
                            }
                            if ((recvbyte == '+') && (gHStartSetup == 1))  {
                                gHStartSetup = 2;
                                gHGUARDTIME = 400;
                                sendEtherByte('+');
                            }
                            if ((recvbyte=='+')&&(gHStartSetup==0)&&  \
                                                            (gHGUARDTIME==0)) {
                                gHStartSetup = 1;
                                gHGUARDTIME = 400;
                                sendEtherByte('+');
                            }
                            if (gSetupMode == 0) {
                                sendB(recvbyte);
                                if (guConfigured)  sendUsbByte(recvbyte);
                                if (gHStartSetup == 0) gHGUARDTIME = 1000;
                            }
                        }
                    }
                    else  {
                        sendB(recvbyte);
                        if (guConfigured) enCodeMidi(recvbyte);

                    }
                }
            }
            recvword = pullUsbRxBuf();
            if (recvword)  {
                recvbyte = (uint8_t)(recvword & 0x00FF);
                sendEtherByte(recvbyte);
            }
        }
        else {    //USB_MASTER
            if (guConfigured && (fillstate < (USART_TXBUFLEN-40))) {
                recvword = pullUsbRxBuf();
                if (recvword)  {
                    recvbyte = (uint8_t)(recvword & 0x00FF);
                    if (guDeviceClass == CDC_CLASS) {
                        if (gStartSetup == 0)  {
                            if ((gHStartSetup == 4) && (gHGUARDTIME)) {
                                gHStartSetup = 0;
                            }
                            if((recvbyte == '+') && (gHStartSetup == 2))  {
                                gHStartSetup = 3;
                                gHGUARDTIME = 1000;      //milliseconds
                                sendUsbByte('+');
                            }
                            if ((recvbyte == '+') && (gHStartSetup == 1))  {
                                gHStartSetup = 2;
                                gHGUARDTIME = 400;
                                sendUsbByte('+');
                            }
                            if ((recvbyte=='+')&&(gHStartSetup==0)&&  \
                                                            (gHGUARDTIME==0)) {
                                gHStartSetup = 1;
                                gHGUARDTIME = 400;
                                sendUsbByte('+');
                            }
                            if (gSetupMode == 0) {
                                sendB(recvbyte);
                                sendEtherByte(recvbyte);
                                if (gHStartSetup == 0) gHGUARDTIME = 1000;
                            }
                        }
                    }
                    else  {
                        sendB(recvbyte);
                        sendEtherByte(recvbyte);
                    }
                }
            }
            if (geEtherOpen) {
                recvword = pullSpiRxBuf();
                if (recvword)  {
                    recvbyte = (uint8_t)(recvword & 0x00FF);
                    if (guDeviceClass == CDC_CLASS) sendUsbByte(recvbyte);
                    else  enCodeMidi(recvbyte);
                }
            }
        }
    }
}


//##########################################################################
// Misc Low Level Routines
//************************

void msDelay(uint16_t count)   // *******************************************
//blocking delay by 'count' ms
// **************************************************************************
{
    gDELAYCOUNT = count;
    while(gDELAYCOUNT) IWDG->KR = 0x0000AAAA;
                                    //feed watchdog, timeout ca. 1-2 seconds;
}


void servSystem(void)   // ************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called in main loop and pullUsbRxBuf()
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally slowed down by SysTick to 50 times/second
// ****************************************************************************
{
uint32_t classChange;
static uint8_t secCount, temp;

//system relevant, but not extremely time critical services
//are handled quasi in the background

//high speed part:  *****************************
//handle SPI Tx Query every 1ms to get a good balance
//between latency of MIDI channel messages and fast transfer of bulk data
    if (gStrobe1ms)  {
        gStrobe1ms = 0;
        if (geEtherOpen)  checkSendEther();
    }

    if (geEtherMode != 2) receiveUDP();
    else{
        receiveTCP();
        temp = readWiz_4(S0_IR,S0_REGBLOCK); //serve TCP related interrupt only
        if (temp & 0x08)  openTCP0();               //TIMEOUT interrupt
                                                    //re-init TCP
        else if ((temp & 0x15) != 0)  {             //SEND,RECV or CON interrupt
            writeWiz_4(S0_IR,S0_REGBLOCK,0xFF);     //ok, clear flag
            if (temp & 1)  {                //CON interrupt, SOCK_ESTABLISHED
                geDiscount = gComParamShadow.DisconTime;
                geEtherOpen = 1;
            }
        }
        else if (temp & 0x02) {                     //DISCON interrupt
            temp = readWiz_4(S0_SR,S0_REGBLOCK);    //check status
            openTCP0();                             //re-init TCP
        }
    }


//lower speed services:  **********************************

    if (gSYSCOUNT == 0)  {
        gSYSCOUNT = 20;         //restart 50Hz service interval

       IWDG->KR = 0x0000AAAA;  //feed watchdog - timeout ca. 1-2 seconds

//LED control
        if (gRED_LEDCOUNT==0)  GPIOA->BSRR =  GPIO_BSRR_BS_0;
                                                    //PA0 (red LED) ON again
        if (gGREEN_LEDCOUNT==0)  GPIOA->BSRR =  GPIO_BSRR_BS_1;
                                                    //PA1 (green LED) ON again

#ifndef DEBUG
//very low speed service (1 per sec)    *******************
        secCount ++;
        if (secCount >= 50)  {
            secCount = 0;
//check panel switch or jumper and force new USB enumeration if necessary
//switch open: = (default) Artnet and parallel ASCII commands, Mini DMX via USB
//switch closed: = Artnet and parallel MIDI commands via USB
            classChange = 0;
            if (GPIOA->IDR & GPIO_IDR_14)  {        //then switch is open
                if (guDeviceClass == MIDI_CLASS) {
                    stopUSB();
                    guDeviceClass = CDC_CLASS;
                    classChange = 0xFF;
                }
            }
            else if (guDeviceClass == CDC_CLASS) {
                stopUSB();
                guDeviceClass = MIDI_CLASS;
                classChange = 0xFF;
            }
            if (classChange)  {
                GPIOA->BSRR =  GPIO_BSRR_BR_0;      //PA0 (red LED) OFF
                GPIOA->BSRR =  GPIO_BSRR_BR_1;      //PA1 (green LED) OFF
                startUsb();
                GPIOA->BSRR =  GPIO_BSRR_BS_0;      //PA0 (red LED) ON again
                GPIOA->BSRR =  GPIO_BSRR_BS_1;      //PA1 (green LED) ON

                if (guDeviceClass==CDC_CLASS) geEtherMode =gComParam.LanMode &3;
                else  geEtherMode = (gComParamShadow.LanMode >> 4) & 3;
                        //if MIDI_CLASS, geEthermode may be changed on the fly
                if (geEtherMode == 2) openTCP0();
                else openUDP0();
                msDelay(5);
                initUsart2();                       //Baud Rate may be changed
                msDelay(5);
            }

            gStreamMaster = ETHER_MASTER;
            if ((GPIOA->IDR & m13) == 0)  gStreamMaster = USB_MASTER;

            if (geDiscount != 0)  {                 //TCP disconnect timer
                geDiscount--;
                if ((geDiscount == 5) && (geEtherMode == 2))  {
                    sendEtherByte(0x20);
                    sendEtherByte('E');
                    sendEtherByte('X');
                    sendEtherByte('5');
                    sendEtherByte('s');
                    sendEtherByte('!');
                    sendEtherByte(0x20);
                    geDiscount = 5;
                }
                if ((geDiscount == 0)&&(geEtherMode == 2)) closeTCPConnection();
            }
        }
#endif
    }
}


void popFlashPage31(void)   //***********************************************
// **************************************************************************
{
uint16_t  *flashPtr, *comParamPtr;
uint32_t k;

    flashPtr = (uint16_t *)PAGE31;
    comParamPtr = (uint16_t *) &(gComParam.VidPid);
    for (k=0;k<34;k++)  {
        *comParamPtr = ~(*flashPtr++);
        comParamPtr++;
    }

//if flash item is empty, initialize default values
    if (gComParam.MacHi == 0) gComParam.MacHi = MACHI_DEFAULT;
    if (gComParam.MacLo == 0) gComParam.MacLo = MACLO_DEFAULT;
    if (gComParam.Gateway == 0) gComParam.Gateway = GATEWAY_DEFAULT;
    if (gComParam.LocalIP == 0) gComParam.LocalIP = LOCALIP_DEFAULT;
    if (gComParam.Subnet == 0) gComParam.Subnet = SUBNET_DEFAULT;
    if (gComParam.LanMode == 0) gComParam.LanMode = LANMODE_DEFAULT;
    if (gComParam.UdpDestIP == 0) gComParam.UdpDestIP = UDESTIP_DEFAULT;
    if (gComParam.UdpDestPort == 0) gComParam.UdpDestPort = UDESTPORT_DEFAULT;
    if (gComParam.UdpLocalPort == 0) gComParam.UdpLocalPort= ULOCALPORT_DEFAULT;
    if (gComParam.McastIP == 0) gComParam.McastIP = MCASTIP_DEFAULT;
    if (gComParam.McastLocalPort == 0) gComParam.McastLocalPort = MCLOCALPORT_DEFAULT;
    if (gComParam.McastDestPort == 0) gComParam.McastDestPort = MCDESTPORT_DEFAULT;
    if (gComParam.TcpPort == 0) gComParam.TcpPort = TCPPORT_DEFAULT;
    if (gComParam.DisconTime == 0) gComParam.DisconTime = DISCONTIME_DEFAULT;
    if (gComParam.ComBaud == 0) gComParam.ComBaud = COMBAUD_DEFAULT;
    if (gComParam.MidiBaud == 0) gComParam.MidiBaud = MIDIBAUD_DEFAULT;


    if (gComParam.VidPid == 0) gComParam.VidPid = VID_DEFAULT<<16 | PID_DEFAULT;

//copy active parameters into shadow for subsequent setup and display
    flashPtr = (uint16_t *) &(gComParamShadow.VidPid);   //temporarily misused
    comParamPtr = (uint16_t *) &(gComParam.VidPid);
    for (k=0;k<34;k++)  {
        *flashPtr++ = *comParamPtr++;
    }
    k = gComParam.LanMode >> 12;
    gBlockF6 = (uint8_t)(k & 4);
    gBlockFE = (uint8_t)(k & 2);
    gBlockF8 = (uint8_t)(k & 1);
    if (guDeviceClass == CDC_CLASS)  geEtherMode = gComParam.LanMode & 3;
    else  geEtherMode = (gComParam.LanMode >> 4) & 3;
}


uint8_t pushFlashPage31(void)   //**********************************************
// **************************************************************************
{
uint16_t  *flashPtr, *comParamPtr, k;
//uint32_t  *testFlashPtr, *shadowPtr;
uint16_t flashtest,  ramtest;
const uint8_t ErrorString[] = {" FLASH ERROR !"};

//unlock flash
#define KEY1    0x45670123              //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while ((FLASH->SR & FLASH_SR_BSY) != 0);  //wait until no operation going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }

//erase Page31
    FLASH->CR |= FLASH_CR_PER;
    FLASH->AR = PAGE31;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    else {
        sendSetupX32(FLASH->SR);                    //better error management ?
        sendSetupB('?');
    }
    FLASH->CR &= ~FLASH_CR_PER;
//burn shadow into flash page31. Will be active after next reset or power cycle
//This processor allows only half-word flash write
    flashPtr = (uint16_t *)PAGE31;
    comParamPtr = (uint16_t *) &(gComParamShadow.VidPid);
    FLASH->CR |= FLASH_CR_PG;
    for (k=0;k<34;k++)  {
        *flashPtr = ~(*comParamPtr++);
        flashPtr++;
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        else {
            sendSetupX32(FLASH->SR);                //better error management ?
            sendSetupB('!');
        }
    }
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);

//check integrity of flash
    flashPtr = (uint16_t *)PAGE31;
    comParamPtr = (uint16_t *) &(gComParamShadow.VidPid);
    for (k=0;k<34;k++)  {
        flashtest = ~(*flashPtr++);
        ramtest = *comParamPtr++;
        if (ramtest != flashtest)  {
            sendSetupString(ErrorString);
            sendSetupB(0x0D);
            return 0;
        }
    }
    gFlashChanged = 0;
    return 1;
}


void initUsart2(void) // ****************************************************
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
uint16_t baudrate, baudset;

    RCC->APB1ENR |=  RCC_APB1ENR_USART2EN ;     // USART2 clock enable

    GPIOA->MODER &= ~(GPIO_MODER_MODER2_0 |     // Set PortA 2,3
                      GPIO_MODER_MODER3_0 ) ;   //in alternate function mode
    GPIOA->MODER |= (GPIO_MODER_MODER2_1 |      // 10=Alternate function mode
                     GPIO_MODER_MODER3_1 ) ;
    //GPIOA->OTYPER is push-pull by default
    //GPIOA->OSPEEDR is 2 MHz by default
    //GPIOA->PUPDR is HiZ by default (except PA13,14,15)

//USART is AF1
    GPIOA->AFR[0] &= 0xFFFF00FF;
    GPIOA->AFR[0] |= 0x00001100;
    GPIOA->AFR[1] = 0x00000000;

    gRXHEAD = gRXBUF;
    gRXTAIL = gRXBUF;
    gTXHEAD = gTXBUF;
    gTXTAIL = gTXBUF;

#ifdef DEBUG
    USART2->BRR =0x00D0;        //115200Baud: 13.0 at 24 MHz APB1 Clock
#else
    if (guDeviceClass == CDC_CLASS) baudrate = (uint16_t)gComParam.ComBaud;
    else  baudrate = (uint16_t)gComParamShadow.MidiBaud; //immediately active !
    switch(baudrate)  {
        case 96:
            baudset = 0x09C0;       //9600Baud: 156.0 at 24 MHz APB1 Clock
            break;
        case 19:
            baudset = 0x04E0;       //19200Baud: 78.0 at 24 MHz APB1 Clock
            break;
        case 31:
            baudset = 0x0300;       //MIDI=31250Baud: 48.0 at 24 MHz APB1 Clock
            break;
        case 38:
            baudset = 0x0270;       //38400Baud: 39.0 at 24 MHz APB1 Clock
            break;
        case 57:
            baudset = 0x01A0;       //57600Baud: 26.0 at 24 MHz APB1 Clock
            break;
        case 11:
            baudset = 0x00D0;       //115200Baud: 13.0 at 24 MHz APB1 Cloc
            break;
        case 23:
            baudset = 0x0068;       //230400Baud: 6.5 at 24 MHz APB1 Clock
            break;
        default:                    //default (virgin flash or undefined else)
            if (guDeviceClass == CDC_CLASS) baudset = 0x00D0;  //115200Baud
            else baudset = 0x0300;                             //MIDI=31250Baud
            break;
    }
#endif
    USART2->BRR = baudset;

    USART2->CR2 = 0;			//default:1 Stopbit
    USART2->CR3 = 0;			//default:no Handshake
    USART2->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |
                  USART_CR1_TE | USART_CR1_RE;  //8Bit, no Parity,RX Int ON
//the TXIE flag will be set when the first byte is going to be transmitted
    NVIC_EnableIRQ(USART2_IRQn);
}


void sendB(uint8_t data) // *************************************************
//send 1 raw byte via USART
// **************************************************************************
{
uint32_t fillstate;

//prevent buffer underflow during setup and large data via Ethernet, USB
    do {
        if (gTXTAIL > gTXHEAD) {
            fillstate = (uint32_t)gTXHEAD + USART_TXBUFLEN - (uint32_t)gTXTAIL;
        }
        else fillstate = (uint32_t)gTXHEAD - (uint32_t)gTXTAIL ;
    }
    while (fillstate > (USART_TXBUFLEN-40));

    *gTXHEAD++ = data;
    if (gTXHEAD == gTXBUFTOP)  gTXHEAD=gTXBUF;
    USART2->CR1 |= USART_CR1_TXEIE;     //if new data:always re-enable interrupt
}


void sendX8(uint8_t data)  // ***********************************************
//send hex representation of one byte as ASCII text
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    sendB(0x20);
}


void sendX16(uint16_t data) // **********************************************
//send hex representation of one half-word as ASCII text
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendNibble(nibble);
    sendB(0x20);
}


void sendNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendB(asciiNibble);
}


void sendAllB(uint8_t data) // **********************************************
// for error message when USB buffer overflow
// **************************************************************************
{
    if (gStreamMaster == ETHER_MASTER)  sendEtherByte(data);
    else if (guConfigured)  sendUsbByte(data);
    sendB(data);
}

void sendAllString(const uint8_t * str) // **********************************************
// for error message when USB buffer overflow
// **************************************************************************
{
uint8_t byte;

    IWDG->KR = 0x0000AAAA;  //feed watchdog - timeout ca. 1-2 seconds
    while  ((byte = *str++) != 0)  sendAllB(byte);
}


void sendSetupB(uint8_t data) // *************************************************
//send 1 raw byte to active setup device
// **************************************************************************
{
//    servSystem();
    if (gSetupMode == 0xFF)  {
        if (gStreamMaster == ETHER_MASTER)  {
            if (geEtherOpen) sendEtherByte(data);
        }
        else if (guConfigured)  sendUsbByte(data);
    }
    else sendB(data);
 //   servSystem();
}


void sendSetupString(const uint8_t * str) // ********************************************
// ***************************************************************************
{
uint8_t byte;

    IWDG->KR = 0x0000AAAA;  //feed watchdog - timeout ca. 1-2 seconds
    while  ((byte = *str++) != 0)  sendSetupB(byte);
}


void sendSetupX8(uint8_t data)  // ********************************************
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendSetupB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendSetupB(temp);
    sendSetupB(0x20);
}


void sendSetupN8(uint8_t data) // *********************************************
// send 1 byte in hex format (without leading 0x), WITHOUT TRAILING SPACE
// **************************************************************************
{
    uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendSetupB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendSetupB(temp);
}


void sendSetupNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendSetupB(asciiNibble);
}


void sendSetupX16(uint16_t data) // *******************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendSetupNibble(nibble);
    sendSetupB(0x20);
}


void sendSetupX32(uint32_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendSetupNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendSetupNibble(nibble);
    sendSetupB(0x20);
}


void sendSetupN32(uint32_t data) // **********************************************
//send signed decimal number string.
//Leading zeroes are suppressed, no trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
uint32_t quotient;

    if (data >= 0x80000000) {
        sendSetupB('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    if (data < 100000) goto TH10;
    if (data < 1000000) goto TH100;
    if (data < 10000000) goto MILLIONS;
    if (data < 100000000) goto MIL10;
    if (data < 1000000000) goto MIL100;
    quotient = data/1000000000;
    data = data - quotient*1000000000;
    sendSetupB((uint8_t)quotient+ 0x30);
MIL100:
    quotient = data/100000000;
    data = data - quotient*100000000;
    sendSetupB((uint8_t)quotient + 0x30);
MIL10:
    quotient = data/10000000;
    data = data - quotient*10000000;
    sendSetupB((uint8_t)quotient + 0x30);
MILLIONS:
    quotient = data/1000000;
    data = data - quotient*1000000;
    sendSetupB((uint8_t)quotient + 0x30);
TH100:
    quotient = data/100000;
    data = data - quotient*100000;
    sendSetupB((uint8_t)quotient+ 0x30);
TH10:
    quotient = data/10000;
    data = data - quotient*10000;
    sendSetupB((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendSetupB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendSetupB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendSetupB((uint8_t)quotient + 0x30);
ONES:
    sendSetupB((uint8_t)data + 0x30) ;
}


void sendSetupS32(uint32_t data) // **********************************************
//send signed decimal number string.
//Leading zeroes are suppressed, no trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendSetupN32(data);
    sendSetupB(0x20);
}


void sendSetupS16(uint16_t data) // ********************************************
//send signed decimal number string. Leading zeroes are suppressed
//up to  +/- 32767
// **************************************************************************
{
uint16_t quotient;

    if (data >= 0x8000) {
        sendSetupB('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    quotient = data/10000;
    data = data - quotient*10000;
    sendSetupB((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendSetupB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendSetupB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendSetupB((uint8_t)quotient + 0x30);
ONES:
    sendSetupB((uint8_t)data + 0x30) ;
}


uint16_t pullUsartRxBuf(void) // ********************************************
// pull 1 byte asynchronously from USART reveive buffer
// check first if something received. If yes then set bit 15. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
volatile uint16_t recv;

    servSystem();               //serve watchdog etc.

    if (gRXTAIL != gRXHEAD)  {
        recv= (uint16_t)((*gRXTAIL++ & 0x00FF)|0x8000);
        if (gRXTAIL == gRXBUFTOP)  gRXTAIL = gRXBUF;

        if (guDeviceClass == MIDI_CLASS) {
            if ((recv == 0x80F8) && (gBlockF8)) return 0;
            if ((recv == 0x80FE) && (gBlockFE)) return 0;
            if ((recv == 0x80F6) && (gBlockF6)) return 0;
            if ((recv != 0x80F8)&&(recv != 0x80FE)) {
            //if MIDI mode don't blink for Timing Clock or Active Sensing
                GPIOA->BSRR = GPIO_BSRR_BR_1;       // green LED OFF
                gGREEN_LEDCOUNT = 200;              //milliseconds
            }
        }
        else   {
            GPIOA->BSRR = GPIO_BSRR_BR_1;           // green LED OFF
            gGREEN_LEDCOUNT = 200;                  //milliseconds
        }
        return recv;
    }
    else return 0;
}


uint16_t pullSetupRxBuf(void)  // *************************************
//receive byte from active setup device (USART or StreamMaster)
// ********************************************************************
{
volatile uint16_t recv;
//Master has priority over UART
    if (gStreamMaster == ETHER_MASTER)  {
        if ((recv=pullSpiRxBuf()) != 0)  return recv;
    }
    else if (guConfigured)  {
        if ((recv=pullUsbRxBuf()) != 0)  return recv;
    }
    if ((recv=pullUsartRxBuf()) != 0) return recv;
    return 0;
}



// ##########################################################################

// Interrupt Handlers
//*******************

void SysTick_Handler(void)  // ***********************************************
// ***************************************************************************
{

//trigger to handle SPI Tx Query every 1ms to get a good balance
//between latency of MIDI channel messages and fast transfer of bulk data
//details see servSystem();
        gStrobe1ms = 1;

//handle general timing
    if ((gSYSCOUNT >0)&&(gSYSCOUNT < 0x8000)) gSYSCOUNT--;//control servSystem()
    else gSYSCOUNT = 0;             //double safetiy against underflow

    if (gRED_LEDCOUNT > 0) gRED_LEDCOUNT--;
    if (gGREEN_LEDCOUNT > 0) gGREEN_LEDCOUNT--;
    if (gDELAYCOUNT > 0) gDELAYCOUNT--;
    if (gTIMEOUT > 0) gTIMEOUT--;   //to be used in any subroutine
    if (gGUARDTIME) gGUARDTIME--;
    else {
        if (gStartSetup == 3) {     //device in setup mode
            gStartSetup = 0xFF;
            gHStartSetup = 0xFF;
            gSetupMode = 0x77;      //semaphor against competitive access
        }
        else {
            gStartSetup = 0;
        }
    }

    if (gHGUARDTIME) gHGUARDTIME--;
    else {
        if (gHStartSetup == 3) {    //host in setup mode
            gStartSetup = 0xFF;
            gHStartSetup = 0xFF;
            gSetupMode = 0xFF;      //semaphor against competitive access
        }
        else {
            gHStartSetup = 0;
        }
    }
}


void USART2_IRQHandler(void)    // *******************************************
//changed 01 Dec 2017 / REVNUM 4: serial TX interrupt driven
// **************************************************************************
{
    if (USART2->ISR & USART_ISR_RXNE)
    {
        *gRXHEAD++ = (uint8_t)(USART2->RDR & 0x00FF);
        if (gRXHEAD==gRXBUFTOP)  gRXHEAD=gRXBUF;
    }
    if (USART2->ISR & USART_ISR_TXE)  {
        if (gTXTAIL == gTXHEAD){                //nothing to transmit
            USART2->CR1 &= ~USART_CR1_TXEIE;    //then switch interrupt off
            return;
        }
        USART2->TDR = *gTXTAIL++ ;
        if (gTXTAIL == gTXBUFTOP)  gTXTAIL=gTXBUF;
    }
}

//##########################################################################

