// user interface specific routines of Ethernet-MIDI-USB converter  ***********
// for STM32F042F6 or STM32F070F6(20pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
// and WIZ850io wired Ethernet module
//
// Author and (c)2018 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.c", "usb-fs.c" and "ether.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz 1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx_modified.h"
#include "globaldefines.h"

// ************************************

extern struct PARAM{
uint32_t VidPid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t Gateway;
uint32_t LanMode;           //includes BlockF68E configuration
uint32_t UdpDestIP;
uint32_t UdpDestPort;
uint32_t UdpLocalPort;
uint32_t McastIP;
uint32_t McastLocalPort;
uint32_t McastDestPort;
uint32_t TcpPort;
uint32_t DisconTime;
uint32_t ComBaud;
uint32_t MidiBaud;
} gComParam, gComParamShadow ;  //17 words

extern uint16_t gVidShadow, gPidShadow;
extern uint32_t gBlockF6, gBlockFE, gBlockF8;
extern uint32_t gStartSetup, gHStartSetup, gSetupMode, gFlashChanged;
extern const uint8_t gTypeString[];
extern const uint8_t gCopyString[];
extern void popFlashPage31(void);
extern uint8_t pushFlashPage31(void);
extern void servSystem(void);
extern void msDelay(uint32_t);

//RS-232
extern void sendB(uint8_t);     //useful for debugging
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendSetupB(uint8_t);
extern void sendSetupString(const uint8_t *);
extern void sendSetupX8(uint8_t);
extern void sendSetupN8(uint8_t);
extern void sendSetupX16(uint16_t);
extern void sendSetupX32(uint32_t);
extern void sendSetupS32(uint32_t);
extern void sendSetupN32(uint32_t);
extern void sendSetupString(const uint8_t *);
uint16_t pullSetupRxBuf(void);

//USB
extern uint32_t guDeviceClass;

// ************************************

uint32_t gcEtherIP;
uint8_t gcPushByte, gcSetupMode;

const uint8_t gcSetStr[] = {"set"};
const uint8_t gcVidPidStr[] = {" USB Vid/Pid of this device (hex)"};
const uint8_t gcMacIdStr[] = {" Ethernet MAC ID (hex)m5.m4.m3.m2.m1.m0)"};
const uint8_t gcGatewayStr[] = {" Gateway IP (ip3.ip2.ip1.ip0) "};
const uint8_t gcLocalIpStr[] = {" IP of this device (ip3.ip2.ip1.ip0)"};
const uint8_t gcSubnetStr[] = {" Subnet Mask of this device (m3.m2.m1.m0)"};
const uint8_t gcComLanModeStr[] = {" Lan Mode for Serial COM: 0=UDP Multicast, 1=std.UDP, 2=TCP"};
const uint8_t gcMidiLanModeStr[] = {" Lan Mode for MIDI: 0=UDP Multicast, 1=std.UDP, 2=TCP"};
const uint8_t gcMcastIpStr[] = {" Multicast UDP IP(ip3.ip2.ip1.ip0) must be MCAST compatible!"};
const uint8_t gcMcastLocalPortStr[] = {" Multicast UDP Local(Rx) Port number"};
const uint8_t gcMcastDestPortStr[] = {" Multicast UDP Destination(Tx) Port number"};
const uint8_t gcUdpDestIpStr[] = {" Std.UDP Destination IP incl.Broadcast(ip3.ip2.ip1.ip0)"};
const uint8_t gcUdpLocalPortStr[] = {" UDP Local(Rx) Port number (std.UDP, not Multicast)"};
const uint8_t gcUdpDestPortStr[] = {" UDP Destination(Tx) Port number (std.UDP, not Multicast)"};
const uint8_t gcBlockF68EStr[] = {" Bit0:block TimingClk, Bit1:block ActSens, Bit2:block TuneReq"};
const uint8_t gcTcpPortStr[] = {" TCP (Telnet) Port number"};
const uint8_t gcDisconStr[] = {" TCP Disconnect Time (1 ... 65535 seconds, 0=no auto disconnect)"};
const uint8_t gcComBaudStr[] = {" Serial COM Baudrate: first 2 digits,MIDI=31 (active in RS-232 Mode)"};
const uint8_t gcMidiBaudStr[] = {" MIDI Baudrate: first 2 digits,MIDI=31 (active in MIDI Mode)"};
const uint8_t gcSetupStr[] = {"SETUP>"};

void  userSetup();
uint32_t parseHexByte(void);
uint32_t parseHexNibble(void);
uint32_t parseDeciNum(uint32_t);
uint8_t parseDeciNibble(void);
void sendS8(uint32_t);
void sendS16(uint32_t);
uint8_t setVidPid(void);
uint8_t setMacID(void);
uint8_t setEtherIP(void);
void setLanMode(uint32_t);
void setSplitMidi4(void);
void setSplitMidi5(void);
void setBlockF68E(void);
void setBaudrate(uint8_t);
void setMidiChannel(void);
void copyComParams(void);
void sendCRLF(void);
void listCommands(void);
void listSetup(void);


void userSetup(void)  // ***********************************************
// **************************************************************************
{
uint16_t recvword;
uint8_t recvbyte;
//uint16_t * fMirrorPtr;
uint32_t parameter, k;
const uint8_t saveQuestString[] = {"Setup has changed. Save with '|', else changes are lost"};
const uint8_t SaveAgainStr[] = {"Save to Flash?->press again "};
const uint8_t LoadAgainStr[] = {"Reload Setup from Flash?->press again "};
const uint8_t NotSavedStr[] = {" NOT SAVED! "};
const uint8_t NotLoadedStr[] = {" NOT LOADED! "};
const uint8_t ErrorStr[] = {" Flash Write Error, do Setup again! "};

    gFlashChanged = 0;
    gcPushByte = 0;

    sendCRLF();
    k = 0;
    do  {
        recvbyte = gTypeString[k];
        sendSetupB(recvbyte);
        k++;
    } while(recvbyte);
    sendSetupB(HARDWARE_VERSION+0x30);
    sendSetupB('.');
    sendSetupB(REVNUM+0x30);
    k = 0;
    do  {
        recvbyte = gCopyString[k];
        sendSetupB(recvbyte);
        k++;
    } while(recvbyte);
    recvbyte = 0;

    while (1)   {
        sendCRLF();
        sendSetupB('C');
        sendSetupB('M');
        sendSetupB('D');
        sendSetupB('>');
        if (gcPushByte)  {
            recvbyte = gcPushByte;      //uses next command code as
            gcPushByte = 0;             //flexible parameter input delimiter
        }
        else  {
            while ((recvword = pullSetupRxBuf()) == 0) ;
            recvbyte = (uint8_t)(recvword & 0x00FF);
        }

        if ((recvbyte > 0x60) && (recvbyte < 0x7B)) recvbyte -= 0x20;
                                                //force uppercase letter
        if (recvbyte > 0x20)  {
            sendSetupB(recvbyte);
            sendSetupB(':');
            sendSetupB(0x20);
        }

        switch (recvbyte)  {
            case '!':                   //list actual configuration
                listSetup();
                break;

            case 'A':
                sendSetupString(gcSetStr);
                sendSetupString(gcMacIdStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if (setMacID()) gFlashChanged = 1;
                break;

            case 'G':
                sendSetupString(gcSetStr);
                sendSetupString(gcGatewayStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if (setEtherIP())  {
                    gComParamShadow.Gateway = gcEtherIP;
                    gFlashChanged = 1;
                }
                break;

            case 'S':
                sendSetupString(gcSetStr);
                sendSetupString(gcSubnetStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if (setEtherIP())  {
                    gComParamShadow.Subnet = gcEtherIP;
                    gFlashChanged = 1;
                }
                break;

                case 'I':
                sendSetupString(gcSetStr);
                sendSetupString(gcLocalIpStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if (setEtherIP())  {
                    gComParamShadow.LocalIP = gcEtherIP;
                    gFlashChanged = 1;
                }
                break;

                case 'M':
                sendSetupString(gcSetStr);
                sendSetupString(gcMcastIpStr);
                sendSetupB(':');
                sendSetupB(0x20);
                 if (setEtherIP())  {
                    gComParamShadow.McastIP = gcEtherIP;
                    gFlashChanged = 1;
                }
                break;

                case 'H':
                sendSetupString(gcSetStr);
                sendSetupString(gcMcastLocalPortStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536) {
                    gComParamShadow.McastLocalPort = parameter;
                    gFlashChanged = 1;
                }
                break;

                case 'D':
                sendSetupString(gcSetStr);
                sendSetupString(gcMcastDestPortStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536) {
                    gComParamShadow.McastDestPort = parameter;
                    gFlashChanged = 1;
                }
                break;

            case 'U':
                sendSetupString(gcSetStr);
                sendSetupString(gcUdpDestIpStr);
                sendSetupB(':');
                sendSetupB(0x20);
                 if (setEtherIP())  {
                    gComParamShadow.UdpDestIP = gcEtherIP;
                    gFlashChanged = 1;
                }
                break;

            case 'T':
                sendSetupString(gcSetStr);
                sendSetupString(gcUdpDestPortStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536)  {
                    gComParamShadow.UdpDestPort = parameter;
                    gFlashChanged = 1;
                }
                break;

            case 'R':
                sendSetupString(gcSetStr);
                sendSetupString(gcUdpLocalPortStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536)  {
                    gComParamShadow.UdpLocalPort = parameter;
                    gFlashChanged = 1;
                }
                break;

           case 'F':
                sendSetupString(gcSetStr);
                sendSetupString(gcBlockF68EStr);
                sendSetupB(':');
                sendSetupB(0x20);
                setBlockF68E();
                gFlashChanged = 1;
                break;

            case 'L':
                sendSetupString(gcSetStr);
                sendSetupString(gcMidiLanModeStr);
                sendSetupB(':');
                sendSetupB(0x20);
                setLanMode(1);
                gFlashChanged = 1;
                break;

            case 'C':
                sendSetupString(gcSetStr);
                sendSetupString(gcComLanModeStr);
                sendSetupB(':');
                sendSetupB(0x20);
                setLanMode(0);
                gFlashChanged = 1;
                break;

            case 'P':
                sendSetupString(gcSetStr);
                sendSetupString(gcTcpPortStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536)  {
                    gComParamShadow.TcpPort = parameter;
                    gFlashChanged = 1;
                }
                break;

           case 'X':
                sendSetupString(gcSetStr);
                sendSetupString(gcDisconStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if ((parameter = parseDeciNum(65535)) < 65536)  {
                    gComParamShadow.DisconTime = parameter;
                    gFlashChanged = 1;
                }
                break;

           case 'B':
                sendSetupString(gcSetStr);
                sendSetupString(gcComBaudStr);
                sendSetupB(':');
                sendSetupB(0x20);
                setBaudrate(0);
                gFlashChanged = 1;
                break;

           case 'N':
                sendSetupString(gcSetStr);
                sendSetupString(gcMidiBaudStr);
                sendSetupB(':');
                sendSetupB(0x20);
                setBaudrate(1);
                gFlashChanged = 1;
                break;

            case 'V':
                sendSetupString(gcSetStr);
                sendSetupString(gcVidPidStr);
                sendSetupB(':');
                sendSetupB(0x20);
                if (setVidPid() == 0) sendSetupB('?');
                else gFlashChanged = 1;
                break;

            case '?':
                sendCRLF();
                listCommands();
                break;

            case '@':
                msDelay(300);
                if (pullSetupRxBuf() != 0) {
                sendSetupB('?');        //pause <300ms too short
                sendSetupB(0x20);
                }
                sendSetupString(LoadAgainStr);
                do  {
                    recvword = pullSetupRxBuf();
                }  while (recvword == 0) ;
                if (recvword == 0x8040)  {              //'@'
                        popFlashPage31();
                        sendSetupB(0x20);
                        sendSetupB('o');
                        sendSetupB('k');
                        gFlashChanged = 0;
                }
                else  sendSetupString(NotLoadedStr);
                break;

            case '|':
                msDelay(300);
                if (pullSetupRxBuf() != 0) {
                sendSetupB('?');        //pause <300ms too short
                sendSetupB(0x20);
                }
                sendSetupString(SaveAgainStr);
                do  {
                    recvword = pullSetupRxBuf();
                }  while (recvword == 0) ;
                if (recvword == 0x807C)  {              //'|'
                    if (pushFlashPage31())  {
                        sendSetupB(0x20);
                        sendSetupB('o');
                        sendSetupB('k');
                    }
                    else sendSetupString(ErrorStr);
                }
                else  sendSetupString(NotSavedStr);
                break;

            case 'E':
                if (gFlashChanged) {
                    sendSetupString(saveQuestString);
                    sendCRLF();
                    sendSetupB('>');
                    while ((recvword = pullSetupRxBuf()) == 0) ;
                    if (recvword == 0x807C)  {          //'|'
                        msDelay(300);
                        if (pullSetupRxBuf() != 0) {
                        sendSetupB('?');            //pause <300ms too short
                        sendSetupB(0x20);
                        }
                        sendSetupString(SaveAgainStr);
                        do  {
                            recvword = pullSetupRxBuf();
                        }  while (recvword == 0) ;

                        if (recvword == 0x807C)  {      //'|'
                            if (pushFlashPage31())  {
                                sendSetupB(0x20);
                                sendSetupB('o');
                                sendSetupB('k');
                                sendSetupB(0x20);
                            }
                            else sendSetupString(ErrorStr);
                        }
                        else sendSetupString(NotSavedStr);
                    }
                    else  sendSetupString(NotSavedStr);
                }
                sendSetupB('E');
                sendSetupB('x');
                sendSetupB('i');
                sendSetupB('t');
                sendCRLF();

                gSetupMode = 0;
                gStartSetup = 0;
                gHStartSetup = 0;
                gcPushByte = 0;
                gFlashChanged = 0;
                return;

            case 0x0A:
 //               sendSetupB(0x0A);
 //               break;
            case 0x0D:
 //               sendSetupB(0x0D);
                break;
            case 0x20:
                break;      //ignore SPACE
            default:
                sendSetupB('?');
                gcPushByte = 0;
                break;
        }
    }
}


uint32_t parseHexNibble(void)   // ******************************************
// **************************************************************************
{
uint16_t recvword;
uint32_t nibble;

    while ((recvword = pullSetupRxBuf()) == 0) ;
    nibble = (uint32_t)(recvword & 0x00FF);
    if ((nibble == 0x0A)||(nibble == 0x0D)||(nibble == '.')) return 0xFFFFFFFE;
    if  ((nibble >= 0x30) && (nibble <= 0x39))  {
        sendSetupB((uint8_t)nibble);
        return (nibble - 0x30);
    }
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble <= 'F')) {
        sendSetupB((uint8_t)nibble);
        return (nibble - 0x37);
    }
    else return 0xFFFFFFFF;

}


uint32_t  parseHexByte(void)   // ********************************************
// ***************************************************************************
{
uint32_t nibble1, nibble2;

        nibble1 = parseHexNibble();
        if (nibble1 > 0x0000000F) return 0xFFFFFFFF;
        nibble2 = parseHexNibble();
        if (nibble2 > 0x0000000F) return 0xFFFFFFFF;
        return ((nibble1 << 4) | nibble2);
}


uint32_t parseDeciNum(uint32_t limit)   // **********************************
//unsigned, 0 ...max 65535
// **************************************************************************
{
uint32_t decinum;
uint16_t recvword;
uint8_t digit;

    if (limit > 65535) return 0xFFF10000;
    decinum = 0;
    while (decinum <= limit)  {
        while ((recvword = pullSetupRxBuf()) == 0) ;
        digit = (uint8_t)(recvword & 0x00FF);

        if ((digit < 0x30) || (digit > 0x39 )) {
            if (digit > 0x39) gcPushByte = digit;
            //flexible delimiter, will be evaluated as maybe next command code
            return decinum;
        }
        sendSetupB(digit);
        digit -= 0x30;              //ASCII>num
        decinum  = (decinum * 10) + digit;

//avoid necessity of final CRR when max number of places is reached
        if (decinum > limit) {
            gcPushByte = 0;
            return 0xFFFFFFFF;
        }
        if (limit < 10)  {
            gcPushByte = 0;
            return decinum;
        }
        if (decinum >= 10000) {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 1000) && (limit < 10000))  {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 100) && (limit < 1000))  {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 10) &&(limit < 100))  {
            gcPushByte = 0;
            return decinum;
        }
    }
    gcPushByte = 0;
    return decinum;             //exactly == limit
}


uint8_t parseDeciNibble(void)   // ******************************************
// **************************************************************************
{
uint32_t number;

    if ((number = parseDeciNum(15)) > 15) {
        sendSetupB('?');
        return 0xFF;
    }
    return (uint8_t) number;
}


void sendS8(uint32_t number)   // ************************************************
// **************************************************************************
{
    if (number < 100) sendSetupB('0');
    if (number < 10) sendSetupB('0');
    sendSetupN32(number);
}


void sendS16(uint32_t number)   // ************************************************
// **************************************************************************
{
    sendSetupS32(number);
    if (number < 10000) sendSetupB(0x20);
    if (number < 1000) sendSetupB(0x20);
    if (number < 100) sendSetupB(0x20);
    if (number < 10) sendSetupB(0x20);
}


uint8_t setVidPid(void)  // *************************************************
// **************************************************************************
{
uint8_t nibble;
uint16_t Vid, Pid;
const uint8_t VidStr[] = {"Vid="};
const uint8_t PidStr[] = {"Pid="};
    sendSetupString(VidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    sendSetupB(' ');

    sendSetupString(PidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;

//only write to Shadow after all items are entered correctly
    gComParamShadow.VidPid = ((uint32_t)Vid <<16) | (uint32_t)Pid;
    return 1;
}


uint8_t setMacID(void)  // *****************************************************
// **************************************************************************
{
uint32_t  macHi, macLo, temp;

    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = (macHi << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendSetupB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;

// replace not before MAC ID completely parsed
    gComParamShadow.MacHi = macHi;
    gComParamShadow.MacLo = macLo;
    return 1;
}


uint8_t setEtherIP(void)  // ************************************************
// **************************************************************************
{
uint32_t  etherIP, temp;

    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendSetupB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;

// replace not before ID completely parsed
    gcEtherIP = etherIP;
    return 1;
}


void setLanMode(uint32_t commtype)  // **************************************
// **************************************************************************
{
uint32_t mode;
    if ((mode = parseDeciNum(2)) > 2) {
        sendSetupB('?');
        return;
    }
    if (commtype)  {    //MIDI
        gComParamShadow.LanMode &=  0xFFFFFF0F;
        gComParamShadow.LanMode |=  (mode << 4);
    }
    else  {             //serial COM
        gComParamShadow.LanMode &=  0xFFFFFFF0;
        gComParamShadow.LanMode |=  mode;
    }
}


void setBlockF68E(void)  // *************************************************
// **************************************************************************
{
uint32_t mode;

    if ((mode = parseHexNibble()) > 7) {
        sendSetupB('?');
        return;
    }
    gBlockF6 = (uint8_t)(mode & 4);
    gBlockFE = (uint8_t)(mode & 2);
    gBlockF8 = (uint8_t)(mode & 1);
    gComParamShadow.LanMode &= 0xFFFF0FFF;
    gComParamShadow.LanMode |= mode << 12;
}

void setBaudrate(uint8_t mode)  // ******************************************
//mode==0: set ComBaud
//else set MidiBaud
// **************************************************************************
{
uint32_t baudrate;

    if ((baudrate = parseDeciNum(96)) > 96) {
        sendSetupB('?');
        return;
    }
    switch (baudrate) {
        case 11:                                                //115200 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 19:                                                //19200 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 23:                                                //230400 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 31:                                                //31250 = MIDI
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 38:                                                //38400 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 57:                                                //57600 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        case 96:                                                //9600 Baud
            if (mode) gComParamShadow.MidiBaud = baudrate;
            else gComParamShadow.ComBaud = baudrate;
            break;
        default:
            break;
    }
}

void sendCRLF(void)  // *****************************************************
// **************************************************************************
{
    sendSetupB(0x0D);
    sendSetupB(0x0A);
}


void listCommands(void)  // *************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Setup Commands:"};
const uint8_t ActualStr[] = {"list Actual Setup"};
const uint8_t SaveFlashStr[] = {"save new setup permanently (will become active at next power up)"};
const uint8_t LoadFlashStr[] = {"reload setup from Flash (changes since last Power On are cancelled)"};
const uint8_t ExitStr[] = {"E: exit SETUP, return to bidirectional communication"};

    sendCRLF();
    sendCRLF();
    sendSetupString(SetupStr);
    sendCRLF();

    sendSetupB('!');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(ActualStr);
    sendCRLF();

    sendSetupB('V');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcVidPidStr);
    sendCRLF();

    sendSetupB('A');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMacIdStr);
    sendCRLF();

    sendSetupB('G');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcGatewayStr);
    sendCRLF();

    sendSetupB('I');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcLocalIpStr);
    sendCRLF();

    sendSetupB('S');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcSubnetStr);
    sendCRLF();

    sendSetupB('L');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMidiLanModeStr);
    sendCRLF();

    sendSetupB('C');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcComLanModeStr);
    sendCRLF();

    sendSetupB('M');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMcastIpStr);
    sendCRLF();

    sendSetupB('H');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMcastLocalPortStr);
    sendCRLF();

    sendSetupB('D');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMcastDestPortStr);
    sendCRLF();

    sendSetupB('U');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcUdpDestIpStr);
    sendCRLF();

    sendSetupB('R');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcUdpLocalPortStr);
    sendCRLF();

    sendSetupB('T');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcUdpDestPortStr);
    sendCRLF();

    sendSetupB('P');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcTcpPortStr);
    sendCRLF();

    sendSetupB('X');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcDisconStr);
    sendCRLF();

    sendSetupB('F');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcBlockF68EStr);
    sendCRLF();

    sendSetupB('B');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcComBaudStr);
    sendCRLF();

    sendSetupB('N');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(gcSetStr);
    sendSetupString(gcMidiBaudStr);
    sendCRLF();

    sendSetupB('@');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(LoadFlashStr);
    sendCRLF();

    sendSetupB('|');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupString(SaveFlashStr);
    sendCRLF();
    sendSetupString(ExitStr);
}


void listSetup(void)  // *****************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Actual Setup:"};

    sendCRLF();
    sendSetupString(SetupStr);
    sendCRLF();

    sendSetupB('V');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupX16((gComParamShadow.VidPid >>16) & 0x0000FFFF);
    sendSetupB('/');
    sendSetupB(0x20);
    sendSetupX16(gComParamShadow.VidPid & 0x0000FFFF);
    sendSetupString(gcVidPidStr);
    sendCRLF();

    sendSetupB('A');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupN8((gComParamShadow.MacHi>>8) & 0xFF);
    sendSetupB('.');
    sendSetupN8(gComParamShadow.MacHi & 0xFF);
    sendSetupB('.');
    sendSetupN8((gComParamShadow.MacLo >> 24) & 0xFF);
    sendSetupB('.');
    sendSetupN8((gComParamShadow.MacLo >> 16) & 0xFF);
    sendSetupB('.');
    sendSetupN8((gComParamShadow.MacLo >> 8) & 0xFF);
    sendSetupB('.');
    sendSetupN8(gComParamShadow.MacLo & 0xFF);
    sendSetupString(gcMacIdStr);
    sendCRLF();

    sendSetupB('G');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.Gateway >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Gateway >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Gateway >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.Gateway & 0xFF);
    sendSetupString(gcGatewayStr);
    sendCRLF();

    sendSetupB('I');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.LocalIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.LocalIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.LocalIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.LocalIP & 0xFF);
    sendSetupString(gcLocalIpStr);
    sendCRLF();

    sendSetupB('S');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.Subnet >> 24) &0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Subnet >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.Subnet >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.Subnet & 0xFF);
    sendSetupString(gcSubnetStr);
    sendCRLF();

    sendSetupB('L');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16((gComParamShadow.LanMode >> 4) & 0x00000003);
    sendSetupString(gcMidiLanModeStr);
    sendCRLF();

    sendSetupB('C');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.LanMode & 0x00000003);
    sendSetupString(gcComLanModeStr);
    sendCRLF();

    sendSetupB('M');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.McastIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.McastIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.McastIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.McastIP & 0xFF);
    sendSetupString(gcMcastIpStr);
    sendCRLF();

    sendSetupB('H');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.McastLocalPort & 0x0000FFFF);
    sendSetupString(gcMcastLocalPortStr);
    sendCRLF();

    sendSetupB('D');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.McastDestPort & 0x0000FFFF);
    sendSetupString(gcMcastDestPortStr);
    sendCRLF();

    sendSetupB('U');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS8((gComParamShadow.UdpDestIP >> 24) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.UdpDestIP >> 16) & 0xFF);
    sendSetupB('.');
    sendS8((gComParamShadow.UdpDestIP >> 8) & 0xFF);
    sendSetupB('.');
    sendS8(gComParamShadow.UdpDestIP & 0xFF);
    sendSetupString(gcUdpDestIpStr);
    sendCRLF();

    sendSetupB('R');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.UdpLocalPort & 0x0000FFFF);
    sendSetupString(gcUdpLocalPortStr);
    sendCRLF();

    sendSetupB('T');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.UdpDestPort & 0x0000FFFF);
    sendSetupString(gcUdpDestPortStr);
    sendCRLF();

    sendSetupB('P');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.TcpPort & 0x0000FFFF);
    sendSetupString(gcTcpPortStr);
    sendCRLF();

    sendSetupB('X');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16(gComParamShadow.DisconTime & 0x0000FFFF);
    sendSetupString(gcDisconStr);
    sendCRLF();

    sendSetupB('F');
    sendSetupB(':');
    sendSetupB(0x20);
    sendS16((gComParamShadow.LanMode >> 12) & 0x00000003);
    sendSetupString(gcBlockF68EStr);
    sendCRLF();

    sendSetupB('B');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupB((gComParamShadow.ComBaud/10)  + 0x30);
    sendSetupB((gComParamShadow.ComBaud %10) + 0x30);
    switch (gComParamShadow.ComBaud) {
        case 11:
            sendSetupB('5');
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 19:
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 23:
            sendSetupB('0');
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 31:
            sendSetupB('M');
            sendSetupB('I');
            sendSetupB('D');
            sendSetupB('I');
            break;
        case 38:
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 57:
            sendSetupB('6');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 96:
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            sendSetupB(0x20);
            break;
        default:
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB(0x20);
    }
    sendSetupString(gcComBaudStr);
    sendCRLF();

    sendSetupB('N');
    sendSetupB(':');
    sendSetupB(0x20);
    sendSetupB((gComParamShadow.MidiBaud/10)  + 0x30);
    sendSetupB((gComParamShadow.MidiBaud %10) + 0x30);
    switch (gComParamShadow.MidiBaud) {
        case 11:
            sendSetupB('5');
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 19:
            sendSetupB('2');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 23:
            sendSetupB('0');
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            break;
        case 31:
            sendSetupB('M');
            sendSetupB('I');
            sendSetupB('D');
            sendSetupB('I');
            break;
        case 38:
            sendSetupB('4');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 57:
            sendSetupB('6');
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            break;
        case 96:
            sendSetupB('0');
            sendSetupB('0');
            sendSetupB(0x20);
            sendSetupB(0x20);
            break;
        default:
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB('?');
            sendSetupB(0x20);
    }
    sendSetupString(gcMidiBaudStr);
}

// ##########################################################################
